@@ -168,6 +168,14 @@ seed_release_arg (GITransfer transfer,
 	  {
 	    interface_info = g_type_info_get_interface (type_info);
 
+	    GIInfoType interface_type = g_base_info_get_type (interface_info);
+	    if (interface_type == GI_INFO_TYPE_CALLBACK)
+	      {
+	        /* FIXME: - callback returns are not handled yet */
+	        g_base_info_unref (interface_info);
+	        break;
+	      }
+
 	    gtype =
 	      g_registered_type_info_get_g_type ((GIRegisteredTypeInfo *)
 						 interface_info);
@@ -282,6 +290,21 @@ seed_gi_make_jsarray (JSContextRef ctx,
       ret = (JSValueRef) JSObjectMakeArray (ctx, length, elements, exception);
     }
 
+  if (element_type == GI_TYPE_TAG_GTYPE)
+    {
+      GType* ptr = (GType*)array;
+      length = 0;
+      while (ptr[length])  length++;
+    
+      elements = g_alloca (sizeof (JSValueRef) * length);
+
+      for (i = 0; i < length; ++i)
+          elements[i] = seed_value_from_long (ctx, ptr[i], exception);
+        
+      return (JSValueRef) JSObjectMakeArray (ctx, length, elements, exception);
+
+    }
+
   return ret;
 }
 
@@ -366,7 +389,11 @@ seed_gi_make_array (JSContextRef ctx,
 	*array_p = dblresult;
       }
       break;
+
+#if GOBJECT_INTROSPECTION_VERSION < 0x000900
     case GI_TYPE_TAG_INT:
+#endif
+    case GI_TYPE_TAG_INT32:
       {
 	gint *intresult;
 
@@ -424,9 +451,9 @@ seed_gi_make_array (JSContextRef ctx,
 		    elem = JSObjectGetPropertyAtIndex (ctx,
 						       (JSObjectRef) array,
 						       i, exception);
-		    seed_gvalue_from_seed_value (ctx, elem,
-						 (GType) 0,
-						 &gvalresult[i], exception);
+		    seed_value_to_gvalue (ctx, elem,
+					  (GType) 0,
+					  &gvalresult[i], exception);
 		  }
 		*array_p = gvalresult;
 
@@ -447,11 +474,11 @@ seed_gi_make_array (JSContextRef ctx,
 }
 
 gboolean
-seed_gi_make_argument (JSContextRef ctx,
-		       JSValueRef value,
-		       GITypeInfo * type_info,
-		       GArgument * arg,
-		       JSValueRef * exception)
+seed_value_to_gi_argument (JSContextRef ctx,
+			   JSValueRef value,
+			   GITypeInfo * type_info,
+			   GArgument * arg,
+			   JSValueRef * exception)
 {
   GITypeTag gi_tag = g_type_info_get_tag (type_info);
 
@@ -466,7 +493,54 @@ seed_gi_make_argument (JSContextRef ctx,
   switch (gi_tag)
     {
     case GI_TYPE_TAG_VOID:
+      
+      if (g_type_info_is_pointer (type_info))
+      {
+        GObject *gobject;
+        if (JSValueIsString (ctx, value)) 
+          {
+          /* 
+            things like gio.outputstream.write use void pointers 
+            might need a few other types here.. 
+            not very well memory managed. - should be solved by bytearrays when
+            introspection implements it.  
+          */
+          arg->v_string = seed_value_to_string (ctx, value, exception); 
+          break;
+        }
+        /* gtk_statusicon_position_menu / gtk_menu_popup use the userdata for the Gobject */
+        gobject = seed_value_to_object (ctx, value, exception);
+        if (!gobject)
+          return FALSE;
+
+        arg->v_pointer = gobject;
+      }
       break;
+
+#if GOBJECT_INTROSPECTION_VERSION < 0x000900
+    case GI_TYPE_TAG_LONG:
+      arg->v_long = seed_value_to_long (ctx, value, exception);
+      break;
+    case GI_TYPE_TAG_ULONG:
+      arg->v_ulong = seed_value_to_ulong (ctx, value, exception);
+      break;
+    case GI_TYPE_TAG_INT:
+      arg->v_int = seed_value_to_int (ctx, value, exception);
+      break;
+    case GI_TYPE_TAG_UINT:
+      arg->v_uint = seed_value_to_uint (ctx, value, exception);
+      break;
+    case GI_TYPE_TAG_SIZE:
+      arg->v_size = seed_value_to_size (ctx, value, exception);
+      break;
+    case GI_TYPE_TAG_SSIZE:
+      arg->v_ssize = seed_value_to_ssize (ctx, value, exception);
+      break;
+    case GI_TYPE_TAG_TIME_T:
+      arg->v_long = seed_value_to_time_t (ctx, value, exception);
+      break;
+#endif
+
     case GI_TYPE_TAG_BOOLEAN:
       arg->v_boolean = seed_value_to_boolean (ctx, value, exception);
       break;
@@ -488,30 +562,12 @@ seed_gi_make_argument (JSContextRef ctx,
     case GI_TYPE_TAG_UINT32:
       arg->v_uint32 = seed_value_to_uint (ctx, value, exception);
       break;
-    case GI_TYPE_TAG_LONG:
-      arg->v_long = seed_value_to_long (ctx, value, exception);
-      break;
     case GI_TYPE_TAG_INT64:
       arg->v_int64 = seed_value_to_int64 (ctx, value, exception);
       break;
-    case GI_TYPE_TAG_ULONG:
-      arg->v_ulong = seed_value_to_ulong (ctx, value, exception);
-      break;
     case GI_TYPE_TAG_UINT64:
       arg->v_uint64 = seed_value_to_uint64 (ctx, value, exception);
       break;
-    case GI_TYPE_TAG_INT:
-      arg->v_int = seed_value_to_int (ctx, value, exception);
-      break;
-    case GI_TYPE_TAG_UINT:
-      arg->v_uint = seed_value_to_uint (ctx, value, exception);
-      break;
-    case GI_TYPE_TAG_SIZE:
-      arg->v_size = seed_value_to_size (ctx, value, exception);
-      break;
-    case GI_TYPE_TAG_SSIZE:
-      arg->v_ssize = seed_value_to_ssize (ctx, value, exception);
-      break;
     case GI_TYPE_TAG_FLOAT:
       arg->v_float = seed_value_to_float (ctx, value, exception);
       break;
@@ -527,9 +583,6 @@ seed_gi_make_argument (JSContextRef ctx,
     case GI_TYPE_TAG_GTYPE:
       arg->v_int = seed_value_to_int (ctx, value, exception);
       break;
-    case GI_TYPE_TAG_TIME_T:
-      arg->v_long = seed_value_to_time_t (ctx, value, exception);
-      break;
     case GI_TYPE_TAG_INTERFACE:
       {
 	GIBaseInfo *interface;
@@ -542,8 +595,8 @@ seed_gi_make_argument (JSContextRef ctx,
 
 	arg->v_pointer = NULL;
 
-	if (interface_type == GI_INFO_TYPE_OBJECT
-	    || interface_type == GI_INFO_TYPE_INTERFACE)
+	if (interface_type == GI_INFO_TYPE_OBJECT ||
+	    interface_type == GI_INFO_TYPE_INTERFACE)
 	  {
 	    gobject = seed_value_to_object (ctx, value, exception);
 	    required_gtype =
@@ -584,10 +637,13 @@ seed_gi_make_argument (JSContextRef ctx,
 	    g_base_info_unref (interface);
 	    break;
 	  }
-	else if (interface_type == GI_INFO_TYPE_STRUCT)
+	else if (interface_type == GI_INFO_TYPE_STRUCT ||
+		 interface_type == GI_INFO_TYPE_UNION)
 	  {
 	    if (JSValueIsObjectOfClass (ctx, value, seed_struct_class))
 	      arg->v_pointer = seed_pointer_get_pointer (ctx, value);
+	    else if (JSValueIsObjectOfClass (ctx, value, seed_union_class))
+	      arg->v_pointer = seed_pointer_get_pointer (ctx, value);
 	    else
 	      {
 		GType type =
@@ -601,10 +657,10 @@ seed_gi_make_argument (JSContextRef ctx,
 		else if (type == G_TYPE_VALUE)
 		  {
 		    GValue *gval = g_slice_alloc0 (sizeof (GValue));
-		    seed_gvalue_from_seed_value (ctx,
-						 value,
-						 (GType) NULL,
-						 gval, exception);
+		    seed_value_to_gvalue (ctx,
+					  value,
+					  (GType) NULL,
+					  gval, exception);
 		    arg->v_pointer = gval;
 
 		    g_base_info_unref (interface);
@@ -744,13 +800,30 @@ seed_gi_make_argument (JSContextRef ctx,
 }
 
 JSValueRef
-seed_gi_argument_make_js (JSContextRef ctx,
-			  GArgument * arg, GITypeInfo * type_info,
-			  JSValueRef * exception)
+seed_value_from_gi_argument (JSContextRef ctx,
+			     GArgument * arg, GITypeInfo * type_info,
+			     JSValueRef * exception)
 {
   GITypeTag gi_tag = g_type_info_get_tag (type_info);
   switch (gi_tag)
     {
+
+#if GOBJECT_INTROSPECTION_VERSION < 0x000900
+    case GI_TYPE_TAG_LONG:
+      return seed_value_from_long (ctx, arg->v_long, exception);
+    case GI_TYPE_TAG_ULONG:
+      return seed_value_from_ulong (ctx, arg->v_ulong, exception);
+    case GI_TYPE_TAG_INT:
+      return seed_value_from_int (ctx, arg->v_int32, exception);
+    case GI_TYPE_TAG_UINT:
+      return seed_value_from_uint (ctx, arg->v_uint32, exception);
+    case GI_TYPE_TAG_SSIZE:
+      return seed_value_from_ssize (ctx, arg->v_ssize, exception);
+    case GI_TYPE_TAG_SIZE:
+      return seed_value_from_size (ctx, arg->v_size, exception);
+    case GI_TYPE_TAG_TIME_T:
+      return seed_value_from_time_t (ctx, arg->v_long, exception);
+#endif
     case GI_TYPE_TAG_VOID:
       return JSValueMakeUndefined (ctx);
     case GI_TYPE_TAG_BOOLEAN:
@@ -767,22 +840,10 @@ seed_gi_argument_make_js (JSContextRef ctx,
       return seed_value_from_int (ctx, arg->v_int32, exception);
     case GI_TYPE_TAG_UINT32:
       return seed_value_from_uint (ctx, arg->v_uint32, exception);
-    case GI_TYPE_TAG_LONG:
-      return seed_value_from_long (ctx, arg->v_long, exception);
     case GI_TYPE_TAG_INT64:
       return seed_value_from_int64 (ctx, arg->v_int64, exception);
-    case GI_TYPE_TAG_ULONG:
-      return seed_value_from_ulong (ctx, arg->v_ulong, exception);
     case GI_TYPE_TAG_UINT64:
       return seed_value_from_uint64 (ctx, arg->v_uint64, exception);
-    case GI_TYPE_TAG_INT:
-      return seed_value_from_int (ctx, arg->v_int32, exception);
-    case GI_TYPE_TAG_UINT:
-      return seed_value_from_uint (ctx, arg->v_uint32, exception);
-    case GI_TYPE_TAG_SSIZE:
-      return seed_value_from_ssize (ctx, arg->v_ssize, exception);
-    case GI_TYPE_TAG_SIZE:
-      return seed_value_from_size (ctx, arg->v_size, exception);
     case GI_TYPE_TAG_FLOAT:
       return seed_value_from_float (ctx, arg->v_float, exception);
     case GI_TYPE_TAG_DOUBLE:
@@ -793,8 +854,6 @@ seed_gi_argument_make_js (JSContextRef ctx,
       return seed_value_from_filename (ctx, arg->v_string, exception);
     case GI_TYPE_TAG_GTYPE:
       return seed_value_from_int (ctx, arg->v_int, exception);
-    case GI_TYPE_TAG_TIME_T:
-      return seed_value_from_time_t (ctx, arg->v_long, exception);
     case GI_TYPE_TAG_ARRAY:
       {
 	GITypeInfo *param_type;
@@ -848,7 +907,17 @@ seed_gi_argument_make_js (JSContextRef ctx,
 
 	    return strukt;
 	  }
+	else if (interface_type == GI_INFO_TYPE_CALLBACK)
+	  { 
+	    /* FIXME: return values of type callback are not handled yet. */
+	    g_base_info_unref (interface);
+	    return FALSE;
+	  }
+	/* fall through - other types?? */
+	g_base_info_unref (interface);
+	return FALSE;
       }
+
     case GI_TYPE_TAG_GLIST:
       {
 	GITypeInfo *list_type;
@@ -866,8 +935,8 @@ seed_gi_argument_make_js (JSContextRef ctx,
 
 	    larg.v_pointer = list->data;
 	    ival =
-	      (JSValueRef) seed_gi_argument_make_js (ctx, &larg,
-						     list_type, exception);
+	      (JSValueRef) seed_value_from_gi_argument (ctx, &larg,
+							list_type, exception);
 	    if (!ival)
 	      ival = JSValueMakeNull (ctx);
 	    JSObjectSetPropertyAtIndex (ctx, ret, i, ival, NULL);
@@ -892,8 +961,8 @@ seed_gi_argument_make_js (JSContextRef ctx,
 	  {
 	    larg.v_pointer = list->data;
 	    ival =
-	      (JSValueRef) seed_gi_argument_make_js (ctx, &larg,
-						     list_type, exception);
+	      (JSValueRef) seed_value_from_gi_argument (ctx, &larg,
+							list_type, exception);
 	    if (!ival)
 	      ival = JSValueMakeNull (ctx);
 	    JSObjectSetPropertyAtIndex (ctx, ret, i, ival, NULL);
@@ -901,6 +970,12 @@ seed_gi_argument_make_js (JSContextRef ctx,
 	  }
 	return ret;
       }
+    case GI_TYPE_TAG_ERROR:
+      {
+        JSValueRef ret;
+        seed_make_exception_from_gerror (ctx, &ret, (GError*) arg->v_pointer);
+        return ret;
+      }
 
     default:
       return FALSE;
@@ -993,9 +1068,9 @@ seed_value_from_gvalue (JSContextRef ctx,
 }
 
 gboolean
-seed_gvalue_from_seed_value (JSContextRef ctx,
-			     JSValueRef val,
-			     GType type, GValue * ret, JSValueRef * exception)
+seed_value_to_gvalue (JSContextRef ctx,
+		      JSValueRef val,
+		      GType type, GValue * ret, JSValueRef * exception)
 {
   if (G_IS_VALUE (ret))
     g_value_unset (ret);
@@ -1206,8 +1281,8 @@ seed_gvalue_from_seed_value (JSContextRef ctx,
 					      exception);
 		if (type)	// Prevents recursion.
 		  {
-		    return seed_gvalue_from_seed_value (ctx, val,
-							type, ret, exception);
+		    return seed_value_to_gvalue (ctx, val,
+						 type, ret, exception);
 		  }
 		// TODO: FIXME: Handle better?
 		else
